from ml_rapids import HoeffdingTree
from sklearn.tree import DecisionTreeClassifier
from sklearn.model_selection import KFold
from sklearn.metrics import classification_report
from sklearn.datasets import load_iris
import numpy as np
import time


def test():
    methods = [
        {
            'name': 'Decision Tree (scikit-learn)',
            'ctor': DecisionTreeClassifier
        },
        {
            'name': 'Hoeffding Tree (streamDM)',
            'ctor': HoeffdingTree
        }
    ]

    # Prepare data
    iris = load_iris()
    X = iris.data
    y = iris.target
    print(iris.DESCR)

    for method in methods:
        print('\n' + '=' * 53)
        print(method['name'])
        print('=' * 53 + '\n')

        kf = KFold(n_splits=10)

        t = time.time()
        y_pred = []
        for train_index, test_index in kf.split(X):
            X_train, X_test = X[train_index], X[test_index]
            y_train, y_test = y[train_index], y[test_index]

            # Build model
            learner = method['ctor']()
            learner.fit(X_train, y_train)

            # Predict
            pred = learner.predict(X_test)
            y_pred += pred.tolist()

        print(f'Time: {time.time() - t}\n')

        # Report
        print(classification_report(y, y_pred, target_names=iris.target_names))


if __name__ == '__main__':
    test()



import glob
import setuptools
import os

import numpy

FLAGS = ['-JSON_DLL_BUILD', '-DSTREAMDM_EXPORTS', '-fPIC', '-shared', '-std=c++11',
         '-D_GNU_SOURCE', '-D_FILE_OFFSET_BITS=64', '-D_LARGEFILE_SOURCE64', '-O3', '-DUNIX', '-lpython']

cpp_sources = glob.glob('./code/src/**/*.cpp', recursive=True)

swig_lib = setuptools.Extension(
    name='_streamdm',
    sources=[os.path.join('code', 'src', 'streamdm_wrap.cxx'),
             *cpp_sources,
             ],
    include_dirs=[numpy.get_include(), ],
    extra_compile_args=FLAGS,
)

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='ml-rapids',  # Replace with your own username
    version='0.0.1.3',
    author='Klemen Kenda',
    author_email='klemen.kenda@ijs.si',
    description='Incremental learning written in C++ exposed in Python',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='GPLv3',
    url='https://github.com/JozefStefanInstitute/ml-rapids',
    packages=setuptools.find_packages(),
    setup_requires=['numpy', ],
    install_requires=['scikit-learn', ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'Operating System :: MacOS',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: Unix',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
    ],
    ext_modules=[swig_lib, ],
    extra_compile_args=FLAGS,
    python_requires='>=3.6',
)



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
import _streamdm

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)



import numpy as np

class HoeffdingTree(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):

        self.label_map = {}
        self.label_map_inv = {}


        _streamdm.HoeffdingTree_swiginit(self, _streamdm.new_HoeffdingTree())

    def fit(self, samples, targets):
        indexed_targets = []
        for target in targets:
            if (target not in self.label_map):
                map_len = len(self.label_map)
                self.label_map[target] = map_len
                self.label_map_inv[map_len] = target
            indexed_targets.append(self.label_map[target])
        return _streamdm.HoeffdingTree_fit(self, samples, indexed_targets)



    def predict(self, samples):
        predictions_len = len(samples)
        predictions = _streamdm.HoeffdingTree_predict(self, samples, predictions_len)
        return np.array([self.label_map_inv[p] for p in predictions])



    def export_json(self, file_name):
        return _streamdm.HoeffdingTree_export_json(self, file_name)
    __swig_destroy__ = _streamdm.delete_HoeffdingTree

# Register HoeffdingTree in _streamdm:
_streamdm.HoeffdingTree_swigregister(HoeffdingTree)

class HoeffdingAdaptiveTree(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):

        self.label_map = {}
        self.label_map_inv = {}


        _streamdm.HoeffdingAdaptiveTree_swiginit(self, _streamdm.new_HoeffdingAdaptiveTree())

    def fit(self, samples, targets):
        indexed_targets = []
        for target in targets:
            if (target not in self.label_map):
                map_len = len(self.label_map)
                self.label_map[target] = map_len
                self.label_map_inv[map_len] = target
            indexed_targets.append(self.label_map[target])
        return _streamdm.HoeffdingAdaptiveTree_fit(self, samples, indexed_targets)



    def predict(self, samples):
        predictions_len = len(samples)
        predictions = _streamdm.HoeffdingAdaptiveTree_predict(self, samples, predictions_len)
        return np.array([self.label_map_inv[p] for p in predictions])



    def export_json(self, file_name):
        return _streamdm.HoeffdingAdaptiveTree_export_json(self, file_name)
    __swig_destroy__ = _streamdm.delete_HoeffdingAdaptiveTree

# Register HoeffdingAdaptiveTree in _streamdm:
_streamdm.HoeffdingAdaptiveTree_swigregister(HoeffdingAdaptiveTree)

class NaiveBayes(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):

        self.label_map = {}
        self.label_map_inv = {}


        _streamdm.NaiveBayes_swiginit(self, _streamdm.new_NaiveBayes())

    def fit(self, samples, targets):
        indexed_targets = []
        for target in targets:
            if (target not in self.label_map):
                map_len = len(self.label_map)
                self.label_map[target] = map_len
                self.label_map_inv[map_len] = target
            indexed_targets.append(self.label_map[target])
        return _streamdm.NaiveBayes_fit(self, samples, indexed_targets)



    def predict(self, samples):
        predictions_len = len(samples)
        predictions = _streamdm.NaiveBayes_predict(self, samples, predictions_len)
        return np.array([self.label_map_inv[p] for p in predictions])



    def export_json(self, file_name):
        return _streamdm.NaiveBayes_export_json(self, file_name)
    __swig_destroy__ = _streamdm.delete_NaiveBayes

# Register NaiveBayes in _streamdm:
_streamdm.NaiveBayes_swigregister(NaiveBayes)

class LogisticRegression(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):

        self.label_map = {}
        self.label_map_inv = {}


        _streamdm.LogisticRegression_swiginit(self, _streamdm.new_LogisticRegression())

    def fit(self, samples, targets):
        indexed_targets = []
        for target in targets:
            if (target not in self.label_map):
                map_len = len(self.label_map)
                self.label_map[target] = map_len
                self.label_map_inv[map_len] = target
            indexed_targets.append(self.label_map[target])
        return _streamdm.LogisticRegression_fit(self, samples, indexed_targets)



    def predict(self, samples):
        predictions_len = len(samples)
        predictions = _streamdm.LogisticRegression_predict(self, samples, predictions_len)
        return np.array([self.label_map_inv[p] for p in predictions])



    def export_json(self, file_name):
        return _streamdm.LogisticRegression_export_json(self, file_name)
    __swig_destroy__ = _streamdm.delete_LogisticRegression

# Register LogisticRegression in _streamdm:
_streamdm.LogisticRegression_swigregister(LogisticRegression)

class MajorityClass(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):

        self.label_map = {}
        self.label_map_inv = {}


        _streamdm.MajorityClass_swiginit(self, _streamdm.new_MajorityClass())

    def fit(self, samples, targets):
        indexed_targets = []
        for target in targets:
            if (target not in self.label_map):
                map_len = len(self.label_map)
                self.label_map[target] = map_len
                self.label_map_inv[map_len] = target
            indexed_targets.append(self.label_map[target])
        return _streamdm.MajorityClass_fit(self, samples, indexed_targets)



    def predict(self, samples):
        predictions_len = len(samples)
        predictions = _streamdm.MajorityClass_predict(self, samples, predictions_len)
        return np.array([self.label_map_inv[p] for p in predictions])



    def export_json(self, file_name):
        return _streamdm.MajorityClass_export_json(self, file_name)
    __swig_destroy__ = _streamdm.delete_MajorityClass

# Register MajorityClass in _streamdm:
_streamdm.MajorityClass_swigregister(MajorityClass)

class Perceptron(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):

        self.label_map = {}
        self.label_map_inv = {}


        _streamdm.Perceptron_swiginit(self, _streamdm.new_Perceptron())

    def fit(self, samples, targets):
        indexed_targets = []
        for target in targets:
            if (target not in self.label_map):
                map_len = len(self.label_map)
                self.label_map[target] = map_len
                self.label_map_inv[map_len] = target
            indexed_targets.append(self.label_map[target])
        return _streamdm.Perceptron_fit(self, samples, indexed_targets)



    def predict(self, samples):
        predictions_len = len(samples)
        predictions = _streamdm.Perceptron_predict(self, samples, predictions_len)
        return np.array([self.label_map_inv[p] for p in predictions])



    def export_json(self, file_name):
        return _streamdm.Perceptron_export_json(self, file_name)
    __swig_destroy__ = _streamdm.delete_Perceptron

# Register Perceptron in _streamdm:
_streamdm.Perceptron_swigregister(Perceptron)

class Bagging(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):

        self.label_map = {}
        self.label_map_inv = {}


        _streamdm.Bagging_swiginit(self, _streamdm.new_Bagging())

    def fit(self, samples, targets):
        indexed_targets = []
        for target in targets:
            if (target not in self.label_map):
                map_len = len(self.label_map)
                self.label_map[target] = map_len
                self.label_map_inv[map_len] = target
            indexed_targets.append(self.label_map[target])
        return _streamdm.Bagging_fit(self, samples, indexed_targets)



    def predict(self, samples):
        predictions_len = len(samples)
        predictions = _streamdm.Bagging_predict(self, samples, predictions_len)
        return np.array([self.label_map_inv[p] for p in predictions])



    def export_json(self, file_name):
        return _streamdm.Bagging_export_json(self, file_name)
    __swig_destroy__ = _streamdm.delete_Bagging

# Register Bagging in _streamdm:
_streamdm.Bagging_swigregister(Bagging)



